@@ -131,6 +131,16 @@ static iRangecc prevPathSeg_(const char *end, const char *start) {
     return seg;
 }
 
+void stripUrlPort_String(iString *d) {
+    iUrl parts;
+    init_Url(&parts, d);
+    if (!isEmpty_Range(&parts.port)) {
+        /* Always preceded by a colon. */
+        remove_Block(&d->chars, parts.port.start - 1 - constBegin_String(d),
+                     size_Range(&parts.port) + 1);
+    }
+}
+
 void stripDefaultUrlPort_String(iString *d) {
     iUrl parts;
     init_Url(&parts, d);
@@ -248,6 +258,9 @@ iRangecc urlRoot_String(const iString *d) {
     else {
         iUrl parts;
         init_Url(&parts, d);
+        if (equalCase_Rangecc(parts.scheme, "about")) {
+            return (iRangecc){ constBegin_String(d), parts.path.start };
+        }
         rootEnd = parts.path.start;
     }
     return (iRangecc){ constBegin_String(d), rootEnd };
@@ -681,6 +694,17 @@ const iString *withSpacesEncoded_String(const iString *d) {
     return d;
 }
 
+const iString *withScheme_String(const iString *d, const char *scheme) {
+    iUrl parts;
+    init_Url(&parts, d);
+    if (!equalCase_Rangecc(parts.scheme, scheme)) {
+        iString *repl = collectNewCStr_String(scheme);
+        appendRange_String(repl, (iRangecc){ parts.scheme.end, constEnd_String(d) });
+        return repl;
+    }
+    return d;
+}
+
 const iString *canonicalUrl_String(const iString *d) {
     /* The "canonical" form, used for internal storage and comparisons, is:
        - all non-reserved characters decoded (i.e., it's an IRI)
