@@ -1559,7 +1559,7 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
                     iAssert(xAdvance >= 0);
                     if (wrapMode == word_WrapTextMode) {
                         /* When word wrapping, only consider certain places breakable. */
-                        if ((prevCh == '-' || prevCh == '/') && !isPunct_Char(ch)) {
+                        if ((prevCh == '-' || prevCh == '/' || prevCh == '\\') && !isPunct_Char(ch)) {
                             safeBreakPos = logPos;
                             breakAdvance = wrapAdvance;
                             breakRunIndex = runIndex;
